! from rd_iasi_1c.f90 by Hal Woolf
! $Id: avhrr_bt.f90,v 1.3 2008/03/11 22:51:36 rayg Exp $
! f2py2.5 -c avhrr_bt.f90 -m avhrr_bt

    subroutine avhrr_bt(tab_rad,tab_tb)

! * Convert AVHRR radiance to brightess temperature

    implicit none

    real, parameter :: c1=1.1910659E-05, c2=1.438833
    real :: b1, b2
    real :: rad(5)
    real, intent(in) :: tab_rad(6)
    real, intent(out) :: tab_tb(5)
    integer :: deb_can, ic,j

    real :: val_can(3)=(/ 2687.0,927.2, 837.7/)
    real :: b(3)=(/ 2.07409, 0.55208, 0.34753/)
    real :: c(3)=(/ 0.996577,0.998509 , 0.998947/)

    real, parameter :: pi=3.14
    integer :: no_canvis
    integer :: solar_filter(3)=(/1399,2329,140/)
    real :: refl

    rad(1)=tab_rad(1)
    rad(2)=tab_rad(2)
    rad(4)=tab_rad(5)*100000.   !mW
    rad(5)=tab_rad(6)*100000.

! Which channel 3 is in use?

    if(tab_rad(3) >0 .and. tab_rad(4) ==0) then         ! 1.6 micron
        rad(3)=tab_rad(3)
        no_canvis=3
        deb_can=2
    elseif (tab_rad(3) ==0 .and. tab_rad(4) >0) then       ! 3.7 micron
        rad(3)=tab_rad(4)*100000.
        no_canvis=2
        deb_can=1
    else
        rad(3)=0
        deb_can=2
        no_canvis=2
    endif

    do j=1,no_canvis
        refl=10.*( rad(j) * pi * 100.)/solar_filter(j)
        tab_tb(j)=refl
    enddo

! Convert to tb

    do ic=deb_can,3
        b1=C1*val_can(ic)**3
        b2=C2*val_can(ic)
        if(rad(ic+2) > 0) then
         tab_tb(ic+2)=(b2/alog(1.+b1/rad(ic+2))-b(ic))/c(ic)
        else
         tab_tb(ic+2)=-9999.
        endif
    enddo

    end subroutine avhrr_bt


! extract useful data as brightness temperatures and return arrays
    subroutine avhrr_extract(frac, tab_tb, tab_sd, wgt, gcmean, gcstd)
    implicit none
    real, intent(out) :: frac(7)
    real, intent(out) :: tab_tb(5,7), tab_sd(5,7)
    real, intent(in) :: wgt(7)
    real, intent(in) :: gcmean(6,7), gcstd(6,7)
    integer :: ichan, klus
    real :: avhrrmean(6), avhrrsdev(6)

    do klus = 1, 7
        tab_tb(:,klus) = 0.
        tab_sd(:,klus) = 0.
        frac(klus) = wgt(klus)*100.
        if(frac(klus) > 0.) then
            do ichan = 1, 6
                avhrrmean(ichan) = gcmean(ichan,klus)
                avhrrsdev(ichan) = gcstd(ichan,klus)
            enddo
!               Convert means and standard deviations from radiance to brightness temperature
            call avhrr_bt(avhrrmean(:),tab_tb(:,klus))
            call avhrr_bt(avhrrmean(:)+avhrrsdev(:),tab_sd(:,klus))
            tab_sd(:,klus) = tab_sd(:,klus) - tab_tb(:,klus)
        endif
    enddo

    end subroutine avhrr_extract


! find a free logical unit... that this still has to exist is so painful
    subroutine getunit(iu)
    implicit none
    ! * Get a free logical unit for file access

    integer, intent(out) :: iu
    integer :: i
    logical :: open_iu

    do i = 10,99
        inquire(i,opened = open_iu)
        if( .not. open_iu) exit
    enddo

    iu = i
    return
    
    end subroutine getunit


! open or close the output file - return value is the logical unit used by writeout
    function openclose(iouta, infile, numorb)
    implicit none
    integer openclose
    integer, intent(in) :: iouta
    character(len=200), intent(in):: infile
    integer, intent(in) :: numorb

    character(len=42) :: afile
    character(len=14) :: prefix
    character(len=8)  :: date
    character(len=6)  :: time
    character(len=5)  :: orbit
    character(len=4)  :: dottxt
    character(len=2)  :: metop
    character(len=1)  :: unsc
    integer :: m
    
    data dottxt/'.txt'/
    data prefix/'clusters_metop'/
    data unsc/'_'/
    write(orbit,'(i5.5)') numorb
    
    if (iouta .eq. 0) then
        call getunit(iouta)
        m = index(infile,'M')
        metop = infile(m+1:m+2)
        date = infile(m+4:m+11)
        time = infile(m+12:m+17)
        afile = prefix//metop//unsc//date//unsc//time//unsc//orbit//dottxt        
        open(iouta,file=afile,form='formatted',status='unknown')
        openclose = iouta
    else
        close(iouta)
        openclose = 0
    endif
    
    end function openclose
    
! send the output to a file handle which may or may not be equivalent to a C file descriptor... 
! WALL OF PARAMETERS! FEAR
    subroutine writeout(iouta, line,jpos,ifov,itime,nbclass, frac,tab_tb,tab_sd, satlat,satlon,satzen,satazm,solzen,solazm)
    implicit none

    !f2py intent(in) iouta, line,jpos,ifov,itime,nbclass, frac,tab_tb,tab_sd, satlat,satlon,satzen,satazm,solzen,solazm

    integer, intent(in) :: iouta
    integer, intent(in) :: itime, nbclass, line
    integer, intent(in) :: ifov, jpos
    real, intent(in) :: satlat, satlon, satzen, satazm, solzen, solazm
    real, intent(in) :: frac(7)
    real, intent(in) :: tab_tb(5,7), tab_sd(5,7)

    write(iouta,'(3i4,i12,i4,7f7.2,35f8.3,35f8.3,6f10.3)') &
          line, jpos, ifov, itime, nbclass,                &
          frac, tab_tb, tab_sd,                            &
          satlat, satlon, satzen, satazm, solzen, solazm

    end subroutine writeout
